<?php
namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\IterableApi;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Log;

class SendEditOrderMail extends Job
{
    use InteractsWithQueue, Queueable;

    private $data;
    private $webhookType;
    private $api;
    private $iterableObj;
    private $htmlPublicPath;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($data, $webhookType) {
        
        $this->data = $data->data;
        $this->webhookType = $webhookType;
        $this->htmlPublicPath = dirname(__FILE__) . "/../../public/orderEditMailHTML/";
    }

    /**
     * Process order edit notification
     */
    public function handle() {

        $this->iterableObj = new IterableApi();

        $options = new Options();
        $options->setVersion(env('SHOPIFY_API_VER_DEV'));

        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session(env('SHOPIFY_URL_DEV'), env('SHOPIFY_API_PASSWORD_DEV')));

        if (empty($this->webhookType)) {

            Log::channel('sendOrderEditMailLog')->info("The Webhook topic in the webhook response is empty(Request: ".$this->data.")");
            return false;
        }
        
        $orderData = json_decode($this->data);

        $orderId = $orderData->id;
        Log::channel('sendOrderEditMailLog')->info("Processing order - " . $orderId);

        if (trim(strtolower($this->webhookType)) == "orders/create") {

            $metaFieldCreatedResponse = $this->createOrderMetaField($orderData);
            return false;
        }
        
        $emailId = $orderData->email;
        $orderName = $orderData->name;
        $cancelledAt = $orderData->cancelled_at;

        Log::channel('sendOrderEditMailLog')->info("------orders/updated--------");

        $cancelledOrder = $this->isOrderCancelled($cancelledAt);

        if ($cancelledOrder == true) {
            Log::channel('sendOrderEditMailLog')->info("The order is cancelled and no need to send the order edit mail - " . $orderId);
            return false;
        }

        if (empty($orderData->line_items) || !isset($orderData->line_items)) {

            Log::channel('sendOrderEditMailLog')->info("Not found any line item properties in the order
            (OrderId: ".$orderId.")");
            return false;
        }

        $webhookData = [];
        foreach ($orderData->line_items as $orderLineItem) {

            if (!isset($webhookData[$orderLineItem->variant_id])) {

                $webhookData[$orderLineItem->variant_id]['product_title'] = $orderLineItem->title;
                $webhookData[$orderLineItem->variant_id]['variant_title'] = $orderLineItem->variant_title;
                $webhookData[$orderLineItem->variant_id]['quantity'] = $orderLineItem->quantity;
                $webhookData[$orderLineItem->variant_id]['variant_id'] = $orderLineItem->variant_id;
                $webhookData[$orderLineItem->variant_id]['price'] = $orderLineItem->price;
                $webhookData[$orderLineItem->variant_id]['product_id'] = $orderLineItem->product_id;
            }
        }

        Log::channel('sendOrderEditMailLog')->info("Webhook order:".json_encode($webhookData));

        $refundOrder = [];
        if (!empty($orderData->refunds[0]) && isset($orderData->refunds[0])) {

            foreach($orderData->refunds as $refundData) {

                if (!empty($refundData->refund_line_items[0])) {

                    foreach ($refundData->refund_line_items as $lineKey => $lineValue) {

                        if (!isset($refundOrder[$lineValue->line_item->variant_id])) {

                            $refundOrder[$lineValue->line_item->variant_id]['product_title'] = $lineValue->line_item->title;
                            $refundOrder[$lineValue->line_item->variant_id]['variant_title'] = $lineValue->line_item->variant_title;
                            $refundOrder[$lineValue->line_item->variant_id]['product_id'] = $lineValue->line_item->product_id;
                            $refundOrder[$lineValue->line_item->variant_id]['variant_id'] = $lineValue->line_item->variant_id;
                            $refundOrder[$lineValue->line_item->variant_id]['quantity'] = $lineValue->quantity;
                            $refundOrder[$lineValue->line_item->variant_id]['refundTaxAmount'] = number_format((float)$lineValue->total_tax, 2, '.', '');
                            $refundOrder[$lineValue->line_item->variant_id]['refundSubtotalAmount'] = number_format((float)$lineValue->subtotal, 2, '.', '');

                        } else {

                            $refundOrder[$lineValue->line_item->variant_id]['quantity'] = 
                            (int)$refundOrder[$lineValue->line_item->variant_id]['quantity'] + (int)$lineValue->quantity;
                            $refundOrder[$lineValue->line_item->variant_id]['refundTaxAmount'] = 
                            (float)$refundOrder[$lineValue->line_item->variant_id]['refundTaxAmount'] + (float)$lineValue->total_tax;
                            $refundOrder[$lineValue->line_item->variant_id]['refundTaxAmount'] = 
                            number_format((float)$refundOrder[$lineValue->line_item->variant_id]['refundTaxAmount'], 2, '.', '');
                            $refundOrder[$lineValue->line_item->variant_id]['refundSubtotalAmount'] = 
                            (float)$refundOrder[$lineValue->line_item->variant_id]['refundSubtotalAmount'] + (float)$lineValue->subtotal;
                            $refundOrder[$lineValue->line_item->variant_id]['refundSubtotalAmount'] = 
                            number_format((float)$refundOrder[$lineValue->line_item->variant_id]['refundSubtotalAmount'], 2, '.', '');
                        }
                    }
                }
            }

            Log::channel('sendOrderEditMailLog')->info("Refund order:".json_encode($refundOrder));
        }

        $orderMetaFieldDetails = $this->getOrderItemsMetaFieldDetails($orderId);
        

        if (empty($orderMetaFieldDetails['id']) || empty($orderMetaFieldDetails['value'])) {

            Log::channel('sendOrderEditMailLog')->info("Not found the order items meta field in the order(OrderId: ".$orderId.")");
            return false;
        }

        $metaValueData = !empty($orderMetaFieldDetails['value']) ? json_decode($orderMetaFieldDetails['value']) : "";
        $orderMetaId = !empty($orderMetaFieldDetails['id']) ? $orderMetaFieldDetails['id'] : "";
        $initialOrder = [];
        foreach ($metaValueData as $metaKey => $metaValue) {

            $initialOrder[$metaKey]['variant_id'] = $metaKey;
            $initialOrder[$metaKey]['quantity'] = $metaValue;
        }

        $currentOrder = [];

        $currentOrder = $this->getCurrentOrderDetails($webhookData, $refundOrder, $orderId);


        if (!isset($initialOrder) || !isset($currentOrder)) {

            if (!isset($initialOrder)) {

                Log::channel('sendOrderEditMailLog')->info("Initial order details are empty(OrderId: ".$orderId.")");
            }
            
            if (!isset($currentOrder)) {

                Log::channel('sendOrderEditMailLog')->info("Current order details are empty(OrderId: ".$orderId.")");
            }
            return false;
        }

        Log::channel('sendOrderEditMailLog')->info("Initial order:".json_encode($initialOrder));

        Log::channel('sendOrderEditMailLog')->info("Current order:".json_encode($currentOrder));

        $sendMailStatus = 0;
        if (count($initialOrder) != count($currentOrder)) {

            $sendMailStatus = 1;
        } else {

            foreach ($currentOrder as $currentKey => $currentValue) {

                $productDifference = 1;
                $quantityDifference = 1;
                foreach ($initialOrder as $initialKey => $initialValue) {

                    if (trim($currentKey) == trim($initialKey)) {

                        $productDifference = 0;
                        if (trim($currentValue['quantity']) == trim($initialValue['quantity'])) {

                            $quantityDifference = 0;
                        }
                    }
                }

                if ($productDifference == 1 || $quantityDifference == 1) {

                    $sendMailStatus = 1;
                }
            }
        }

        Log::channel('sendOrderEditMailLog')->info("Mail send status: ".$sendMailStatus);

        if ($sendMailStatus == 1) {

            $mailStatus = $this->sendEditMail($webhookData, $initialOrder, $currentOrder, $orderData, $refundOrder);
            if ($mailStatus == "success") {

                Log::channel('sendOrderEditMailLog')->info("Process order metafield update:");
                $metaFieldUpdatedResponse = $this->updateOrderMetaField($currentOrder, $orderId, $orderMetaId);
            }
            return false;

        } else {

            Log::channel('sendOrderEditMailLog')->info("Both initial order and current order details are the same and no need to 
            send the mail(OrderId:".$orderId.")");
            return false; 
        }

        Log::channel('sendOrderEditMailLog')->info("----------------------------");
    }

    /**
     * Create order metafield with order details 
     */
    public function createOrderMetaField($orderData) {

        $orderId = $orderData->id;

        Log::channel('sendOrderEditMailLog')->info("------orders/created--------");

        if (empty($orderData->line_items) || !isset($orderData->line_items)) {

            Log::channel('sendOrderEditMailLog')->info("Not found any line item properties in the order
            (OrderId: ".$orderId.")");
            return false;
        }

        $currentOrderProductDetails = [];
        foreach ($orderData->line_items as $itemKey => $itemValue) {

            $currentOrderProductDetails[$itemValue->variant_id] = $itemValue->quantity;
        }

        $metaFieldCreateRequest = [];
        $metaFieldCreateRequest = array(
            "order" => array(
                "id" => $orderId,
                "metafields" => array(
                    array(
                        "namespace" => "p80",
                        "key" => "orderItems",
                        "value" => json_encode($currentOrderProductDetails),
                        "type" => "json"
                    )
                )
            )
        );

        $request = $metaFieldCreateRequest;

        $retry = false;
        $try = 1;
        $errorMessage = "must be unique within this namespace on this resource";
        $metaAlreadyExistStatus = 0;
        $metaFieldCreateResponse = [];
        do {
            Log::channel('sendOrderEditMailLog')->info($try . ') Create metafield by orderId, Id:'.$orderId);
            $metaFieldCreateResponse = $this->api->rest(
                'PUT',
                '/admin/api/'.env('SHOPIFY_API_VER_DEV').'/orders/'.$orderId.'.json',
                $request
            );

            if (!isset($metaFieldCreateResponse['body']['order'])) {

                sleep(1);
                $retry = true;
                $try++;
            }

            $metaResponseErrorMessage = !empty($metaFieldCreateResponse['body']['metafields.key'][0]) ? 
            $metaFieldCreateResponse['body']['metafields.key'][0] : "";
            if (!empty($metaResponseErrorMessage)) {
                
                if (trim($metaResponseErrorMessage) == $errorMessage) {

                    $metaAlreadyExistStatus = 1;
                    $retry = false;
                }
            }

        } while ($retry && $try <= 3);

        if ($metaAlreadyExistStatus == 1) {

            Log::channel('sendOrderEditMailLog')->info("OrderItems metafield is already created in Shopify(OrderId: ".$orderId.")");
            return false;
        }

        if (!empty($metaFieldCreateResponse['errors']) || !isset($metaFieldCreateResponse['body']['order'])) {

            $errorMsg = !empty($metaFieldCreateResponse['body']) ? json_encode($metaFieldCreateResponse['body']) : 'Failed to create the metafield';
            Log::channel('sendOrderEditMailLog')->info("Failed to create the metafield in Shopify Order(OrderId: ".$orderId.", 
            Error:".$errorMsg.")");
            return false;
        }

        Log::channel('sendOrderEditMailLog')->info("Successfully created the order metafield in Shopify(OrderId: ".$orderId.")");
        Log::channel('sendOrderEditMailLog')->info("-----------------------------");
    }

    /**
     * Get order items metafield details
     */
    public function getOrderItemsMetaFieldDetails($orderId) {

        $retry = false;
        $try = 1;
        $result = '';
        do {
            Log::channel('sendOrderEditMailLog')->info($try . ') Get metafield details, OrderId:'.$orderId);
            $params = null;
            $metaFieldDetails = $this->api->rest(
                'GET',
                '/admin/api/'.env('SHOPIFY_API_VER_DEV').'/orders/'.$orderId.'/metafields.json',
                $params
            );

            if (!empty($metaFieldDetails['errors'])) {

                sleep(1);
                $retry = true;
                $try++;
            }

        } while ($retry && $try <= 3);

        if (!empty($metaFieldDetails['errors']) || !isset($metaFieldDetails['body']['metafields'][0])) {

            $errorMsg = !empty($metaFieldDetails['body']) ? json_encode($metaFieldDetails['body']) : 'Failed to get the metafield details';
            Log::channel('sendOrderEditMailLog')->info("Failed to get the metafield details from Shopify(OrderId: ".$orderId.", 
            Error:".$errorMsg.")");
            return false;
        }

        $orderItemsMetaKey = "orderItems";
        $orderItemMetaDetails = [];
        foreach ($metaFieldDetails['body']['metafields'] as $metaKey => $metaValue) {


            if (trim(strtolower($metaValue['key'])) == strtolower($orderItemsMetaKey)) {

                $orderItemMetaDetails['id'] = $metaValue['id'];
                $orderItemMetaDetails['value'] = $metaValue['value'];
                break;
            }
        }

        if (empty($orderItemMetaDetails)) {

            Log::channel('sendOrderEditMailLog')->info("Order metafield value is empty(OrderId: ".$orderId.")");
        }

        return $orderItemMetaDetails;
    }

    /**
     * Get current order details
     */
    public function getCurrentOrderDetails($webhookData, $refundOrder, $orderId) {

        $currentOrder = [];
        if (empty($refundOrder) || !isset($refundOrder)) {

            Log::channel('sendOrderEditMailLog')->info("No refund details were found in the order(OrderId: ".$orderId.")");

            foreach ($webhookData as $orderKey => $orderValue) {

                $currentOrder[$orderKey]['product_title'] = $orderValue['product_title'];
                $currentOrder[$orderKey]['variant_title'] = $orderValue['variant_title'];
                $currentOrder[$orderKey]['variant_id'] = $orderValue['variant_id'];
                $currentOrder[$orderKey]['quantity'] = $orderValue['quantity'];
            }
        } else {

            Log::channel('sendOrderEditMailLog')->info("Refund details found in the order(OrderId: ".$orderId.")");

            foreach ($webhookData as $orderKey => $orderValue) {

                $refundStatus = 0;
                foreach ($refundOrder as $refundKey => $refundValue) {

                    if (trim($orderKey) == trim($refundKey)) {

                        $quantityDifference = 1;
                        $quantityDifference = (int)$orderValue['quantity'] - (int)$refundValue['quantity'];
                        if ($quantityDifference == 0) {

                            $refundStatus = 1;
                            break;
                        }
                        $currentOrder[$orderKey]['product_title'] = $orderValue['product_title'];
                        $currentOrder[$orderKey]['variant_title'] = $orderValue['variant_title'];
                        $currentOrder[$orderKey]['variant_id'] = $orderValue['variant_id'];
                        $currentOrder[$orderKey]['quantity'] = (int)$orderValue['quantity'] - (int)$refundValue['quantity'];

                        $refundStatus = 1;
                        break;
                    }
                }

                if ($refundStatus == 0) {

                    $currentOrder[$orderKey]['product_title'] = $orderValue['product_title'];
                    $currentOrder[$orderKey]['variant_title'] = $orderValue['variant_title'];
                    $currentOrder[$orderKey]['variant_id'] = $orderValue['variant_id'];
                    $currentOrder[$orderKey]['quantity'] = $orderValue['quantity'];
                }
            }
        }

        return $currentOrder;
    }

    /**
     * Get removed Lip details
     */
    public function getRemovedLipDetails($currentOrder, $initialOrder, $refundOrder) {

        $removedLipDetails = [];

        foreach ($refundOrder as $refundKey => $refundValue) {

            if (!isset($removedLipDetails[$refundKey])) {

                $removedLipDetails[$refundKey]['product_title'] = $refundValue['product_title'];
                $removedLipDetails[$refundKey]['variant_title'] = $refundValue['variant_title'];
                $removedLipDetails[$refundKey]['product_id'] = $refundValue['product_id'];
                $removedLipDetails[$refundKey]['variant_id'] = $refundValue['variant_id'];
                $removedLipDetails[$refundKey]['quantity'] = $refundValue['quantity'];
                $removedLipDetails[$refundKey]['price'] = $refundValue['refundSubtotalAmount'];
            } else {

                $removedLipDetails[$refundKey]['quantity'] = 
                (int)$removedLipDetails[$refundKey]['quantity'] + (int)$refundValue['product_title'];
                $removedLipDetails[$refundKey]['price'] = 
                (float)$removedLipDetails[$refundKey]['price'] + (float)$refundValue['refundSubtotalAmount'];
                $removedLipDetails[$refundKey]['price'] = 
                number_format((float)$removedLipDetails[$refundKey]['price'], 2, '.', '');
            }
        }

        return $removedLipDetails;
    }

    /**
     * Update order metafield
     */
    public function updateOrderMetaField($currentOrder, $orderId, $orderMetaId) {

        $currentOrderProductDetails = [];
        foreach ($currentOrder as $currentKey => $currentValue) {

            $currentOrderProductDetails[$currentKey] = $currentValue['quantity'];
        }

        $metaFieldUpdateRequest = [];
        $metaFieldUpdateRequest = array(
            "metafield" => array(
                "id" => $orderMetaId,
                "value" => json_encode($currentOrderProductDetails),
                "type" => "json"
            )
        );

        Log::channel('sendOrderEditMailLog')->info("Order meta update request:".json_encode($metaFieldUpdateRequest));

        $request = $metaFieldUpdateRequest;

        $retry = false;
        $try = 1;
        $metaFieldUpdateResponse = [];
        do {
            Log::channel('sendOrderEditMailLog')->info($try . ') Update metafield by Metafield Id, Id:'.$orderId);
            $metaFieldUpdateResponse = $this->api->rest(
                'PUT',
                '/admin/api/'.env('SHOPIFY_API_VER_DEV').'/metafields/'.$orderMetaId.'.json',
                $request
            );
            if (!isset($metaFieldUpdateResponse['body']['metafield'])) {

                sleep(1);
                $retry = true;
                $try++;
            }

        } while ($retry && $try <= 3);

        if (!empty($metaFieldUpdateResponse['errors']) || empty($metaFieldUpdateResponse['body']['metafield'])) {

            $errorMsg = !empty($metaFieldUpdateResponse['body']) ? json_encode($metaFieldUpdateResponse['body']) : 'Failed to update the metafield';
            Log::channel('sendOrderEditMailLog')->info("Failed to update the metafield in Shopify Order(OrderId: ".$orderId.", 
            Error:".$errorMsg.")");
            return false;
        }

        Log::channel('sendOrderEditMailLog')->info("Successfully updated the order metafield in Shopify(OrderId: ".$orderId.")");
    }

    /*
    * To check whether the order is cancelled or not
    */
    public function isOrderCancelled($cancelledAt) {

        if ($cancelledAt !== null) {

            return true;
        } else {

            return false;
        }
    }

    /**
     * Get Shopify product image URL
     */
    public function getImageUrl($productId, $orderId) {

        $imageUrl = null;
        if (!empty($productId)) {

            $retry = false;
            $try = 1;
            $result = '';
            do {
                Log::channel('sendOrderEditMailLog')->info($try . ') Get product details, Id:'.$productId);
                $result = $this->api->rest(
                    'GET',
                    '/admin/api/' . env('SHOPIFY_API_VER_DEV') . '/products/' . $productId . '.json',
                    null
                );

                if (!isset($result['body']['product'])) {

                    sleep(1);
                    $retry = true;
                    $try++;
                }

            } while ($retry && $try <= 3);

            if (!empty($result['errors']) || empty($result['body']['product'])) {

                $errorMsg = !empty($result['body']) ? json_encode($result['body']) : 'Failed to get the product details';
                Log::channel('sendOrderEditMailLog')->info("Failed to get the product details from Shopify
                (OrderId: ".$orderId.", ProductId: ".$productId.", Error:".json_encode($errorMsg).")");
                return null;
            }

            $imageDetails = isset($result['body']['product']['images']) ? $result['body']['product']['images'] : "";
            $imageUrl = isset($imageDetails[0]['src']) ? $imageDetails[0]['src'] : "";
            
        }
        return $imageUrl;
    }

    /**
     * Set line item details
     */
    public function setLineItemDetails($webhookData, $initialOrder, $currentOrder, $orderId) {

        $lineItemContent = '';
        foreach ($currentOrder as $variantId => $variantDetails) {
            
            $productName = '';
            $variantName = '';
            $productQty = '';
            $imageUrl = $this->getImageUrl($webhookData[$variantDetails['variant_id']]['product_id'], $orderId);

            // if (!isset($initialOrder[$variantId])) {

            //     $imageUrl = "https://cdn.shopify.com/s/files/1/0066/7569/3639/products/NM2844L600VITD3GUMMIESfront_compact_cropped.jpg?v=1588875151";
            // }

            $productName = $variantDetails['product_title'];
            $variantName = $variantDetails['variant_title'];
            
            $productQty = $variantDetails['quantity'];
            //$newQty = $currentOrder[$variantId]['quantity'];
            $price = isset($webhookData[$variantId]) ? $webhookData[$variantId]['price'] : "0.00";
            $total = (float)$price * (int)$productQty;
            $total = !empty($total) ? number_format((float)$total, 2, '.', '') : "0.00";

            $lineItemContent .= file_get_contents($this->htmlPublicPath."setLineItems.html");
            $lineItemContent = str_replace('$imageUrl$', $imageUrl, $lineItemContent);
            $lineItemContent = str_replace('$productName$', $productName, $lineItemContent);
            $lineItemContent = str_replace('$productQty$', $productQty, $lineItemContent);
            $lineItemContent = str_replace('$variantName$', $variantName, $lineItemContent);
            $lineItemContent = str_replace('$total$', '$'.$total, $lineItemContent);
        }

        return $lineItemContent;
    }

    /**
     * Set removed line items
     */
    public function setRemovedLineItemsDetails($removedLipDetails, $orderId) {

        $removedOrders = '';
        $removedOrders = file_get_contents($this->htmlPublicPath."removeLineItemsTop.html");

        if (isset($removedLipDetails) && !empty($removedLipDetails)) {

            foreach ($removedLipDetails as $removeKey => $removeValue) {

                $productName = $removeValue['product_title'];
                $variantName = $removeValue['variant_title'];
        
                $productQty = $removeValue['quantity'];
                $total = !empty($removeValue['price']) ? number_format((float)$removeValue['price'], 2, '.', '') : "0.00";
                $imageUrl = $this->getImageUrl($removeValue['product_id'], $orderId);
        
                $removedOrders .= file_get_contents($this->htmlPublicPath."removeLineItemsMiddle.html");
                $removedOrders = str_replace('$imageUrl$', $imageUrl, $removedOrders);
                $removedOrders = str_replace('$productName$', $productName, $removedOrders);
                $removedOrders = str_replace('$productQty$', $productQty, $removedOrders);
                $removedOrders = str_replace('$variantName$', $variantName, $removedOrders);
                $removedOrders = str_replace('$total$', '$'.$total, $removedOrders);
            }
        }

        $removedOrders .= file_get_contents($this->htmlPublicPath."removeLineItemsEnd.html");
        return $removedOrders;
    }

    /**
     * Send order edit mail
     */
    public function sendEditMail($webhookData, $initialOrder, $currentOrder, $orderData, $refundOrder) {

        $orderName = $orderData->name;
        $orderId = $orderData->id;
        $orderTotal = 0;
        $subTotal = 0;
        $totalTax = 0;
        $refundSubTotal = 0;
        $refundTotalTax = 0;

        if (!empty($refundOrder) && isset($refundOrder)) {

            foreach ($refundOrder as $refundKey => $refundValue) {

                $refundSubTotal = (float)$refundSubTotal + (float)$refundValue['refundSubtotalAmount'];
                $refundTotalTax = (float)$refundTotalTax + (float)$refundValue['refundTaxAmount'];
            }

            $orderTotal = (float)$orderData->total_price - (float)$refundSubTotal;
            $orderTotal = !empty($orderTotal) ? number_format((float)$orderTotal, 2, '.', '') : "0.00";
            $subTotal = (float)$orderData->subtotal_price - (float)$refundSubTotal;
            $subTotal = !empty($subTotal) ? number_format((float)$subTotal, 2, '.', '') : "0.00";
            $totalTax = (float)$orderData->total_tax - (float)$refundTotalTax;
            $totalTax = !empty($totalTax) ? number_format((float)$totalTax, 2, '.', '') : "0.00";
        } else {

            $orderTotal = (float)$orderData->total_price;
            $orderTotal = !empty($orderTotal) ? number_format((float)$orderTotal, 2, '.', '') : "0.00";
            $subTotal = (float)$orderData->subtotal_price;
            $subTotal = !empty($subTotal) ? number_format((float)$subTotal, 2, '.', '') : "0.00";
            $totalTax = (float)$orderData->total_tax;
            $totalTax = !empty($totalTax) ? number_format((float)$totalTax, 2, '.', '') : "0.00";
        }

        $orderTotal = '$'.$orderTotal;
        $subTotal = '$'.$subTotal;
        $totalTax = '$'.$totalTax;

        Log::channel('sendOrderEditMailLog')->info("orderTotal: ".$orderTotal.", subTotal:".$subTotal.", totalTax:".$totalTax);
        $address = $this->getAddressContent($orderData);
        $removedLipDetails = [];
        if (!empty($refundOrder) && isset($refundOrder)) {

            $removedLipDetails = $this->getRemovedLipDetails($currentOrder, $initialOrder, $refundOrder);
            Log::channel('sendOrderEditMailLog')->info("RemovedLips: ".json_encode($removedLipDetails));
        }

        $currency = $orderData->currency;
        $orderStatusUrl = $orderData->order_status_url;
        $paymentName = isset($orderData->payment_gateway_names[0]) ? $orderData->payment_gateway_names[0] : '';
        $shippingName = isset($orderData->shipping_lines[0]->title) ? trim($orderData->shipping_lines[0]->title) : '';
        $lineItemDetails = $this->setLineItemDetails($webhookData, $initialOrder, $currentOrder, $orderId);
        $removedDetails = $this->setRemovedLineItemsDetails($removedLipDetails, $orderId);
        $email = $orderData->email;
        
        $iterableMailRequest = array(
            "campaignId" => intval(env('ITERABLE_CAMPAIGN_ID', true)),
            "recipientEmail" => $email,
            "dataFields" => array(
                "orderName" => $orderName,
                "totalPrice" => $orderTotal,
                "shippingAddress" => $address['shippingAddress'],
                "billingAddress" => $address['billingAddress'],
                "subTotal" => $subTotal,
                "totalTax" => $totalTax,
                "currency" => $currency,
                "shippingMethod" => $shippingName,
                "paymentMethod" => $paymentName,
                "lineItemDetails" => $lineItemDetails,
                "removedLineItemDetails" => $removedDetails,
                "orderStatusUrl" => $orderStatusUrl,
            ),
        );

        $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

        $mailTriggeredStatus = '';
        if (strtolower(trim($mailTriggerResponse->code)) == 'success') {

            Log::channel('sendOrderEditMailLog')->info("Successfully sent the order update notification to the user, OrderName: " . $orderName.", Email: ".$email);
            $mailTriggeredStatus = "success";
        } else {

            Log::channel('sendOrderEditMailLog')->info("Failed to send the order update notification to the user, OrderName: " . $orderName.", Email: ".$email);
            Log::channel('sendOrderEditMailLog')->info("Response: ".json_encode($mailTriggerResponse));
            $mailTriggeredStatus = "failed";
        }
        return $mailTriggeredStatus;
    }

    /**
     * Get order shipping and billing address
     */
    public function getAddressContent($orderData) {

        $shippingAddress = isset($orderData->shipping_address) ? $orderData->shipping_address : '';
        $billingAddress = isset($orderData->billing_address) ? $orderData->billing_address : '';

        $shippingAddressContent = '';
        $billingAddressContent = '';

        if (!empty($shippingAddress)) {

            $shippingAddressContent .= '
            <p style="color:#777777;font-size:16px;margin:0;line-height:150%;">' .
            $shippingAddress->first_name . '  ' . $shippingAddress->last_name ;
            
            if (isset($shippingAddress->company) && !empty($shippingAddress->company)) {

             $shippingAddressContent .= '<br />' . $shippingAddress->company;
            }
            
            $shippingAddressContent .= '<br />' . $shippingAddress->address1 . '<br />' . $shippingAddress->province .
            '<br />' . $shippingAddress->country . '<br />' . $shippingAddress->zip . '</p>';
        } else {

            $shippingAddressContent .= '
            <p style="color:#777777;font-size:16px;margin:0;line-height:150%;">' .
                '  <br />  <br /> <br /> <br /> <br /> </p>';
        }

        if (!empty($billingAddress)) {
            $billingAddressContent .= '
            <p style="color:#777777;font-size:16px;margin:0;line-height:150%;">' .
            $billingAddress->first_name . '  ' . $billingAddress->last_name;
            
            if(isset($billingAddress->company) && !empty($billingAddress->company)){
             $billingAddressContent .= '<br />' . $billingAddress->company;
            }
            
            $billingAddressContent .= '<br />' . $billingAddress->address1 . '<br />' . $billingAddress->province .
            '<br />' . $billingAddress->country . '<br />' . $billingAddress->zip . '</p>';
        } else {

            $billingAddressContent .= '
            <p style="color:#777777;font-size:16px;margin:0;line-height:150%;">' .
                '  <br />  <br /> <br /> <br /> <br /> </p>';
        }

        return [
            "billingAddress" => $billingAddressContent,
            "shippingAddress" => $shippingAddressContent,
        ];

    }
}
